#include "../Config.h"
#include "../sys.h"
#include "../bsy.h"
#include "../iphdr.h"
#include "../tools.h"

/*
 * SIGTERM, etc. handler
 */
ULONG _System TermHandler (PEXCEPTIONREPORTRECORD p1,
			    PEXCEPTIONREGISTRATIONRECORD p2,
			    PCONTEXTRECORD p3,
			    PVOID pv)
{
  ULONG ulNestLevel = 0;		    /* Variable tracking nesting of
					     * DosEnterMustComplete calls     */
  static int deinit_done = 0;

  if (!deinit_done)
  {
    DosEnterMustComplete (&ulNestLevel);
    bsy_remove_all ();
    deinit_done = 1;
    DosExitMustComplete (&ulNestLevel);
  }

  if (p1->ExceptionNum == XCPT_SIGNAL)
  {
    char *szSigName = "";

    switch (p1->ExceptionInfo[0])
      {
	case XCPT_SIGNAL_INTR:
	  szSigName = "XCPT_SIGNAL_INTR";
	  break;
	case XCPT_SIGNAL_KILLPROC:
	  szSigName = "XCPT_SIGNAL_KILLPROC";
	  break;
	case XCPT_SIGNAL_BREAK:
	  szSigName = "XCPT_SIGNAL_BREAK";
	  break;
      }
    Log (1, "terminated! (%s)", szSigName);
  }
  return XCPT_CONTINUE_SEARCH;		    /* Exception not resolved... */
}

int set_break_handlers (EXCEPTIONREGISTRATIONRECORD * RegRec)
{
  APIRET rc;
  ULONG ulTimes = 0;			    /* The number of times
					     * DosSetSignalExceptionFocus has
					     * been called by the current
					     * process with flag set to
					     * SIG_SETFOCUS, minus the number
					     * of times it has been called
					     * with flag set to
					     * SIG_UNSETFOCUS. */

  RegRec->prev_structure = 0;
  RegRec->ExceptionHandler = (ERR) TermHandler;
  if ((rc = DosSetExceptionHandler (RegRec)) != NO_ERROR)
  {
    Log (1, "DosSetExceptionHandler error: return code = %u", rc);
    return 0;
  }
  if ((rc = DosSetSignalExceptionFocus (SIG_SETFOCUS, &ulTimes)) != NO_ERROR)
  {
    Log (1, "DosSetSignalExceptionFocus error: return code = %u", rc);
    return 0;
  }
  return 1;
}
