/*

   binkp's frames:

   +---------------------- 0=data block, 1=message(command)
   |                +---- data block size / msg's argument size
   |                |
   7  6543210 76543210
   +-+-------+--------+--- ... ---+
   | |   HI      LO   |           | -- data block / msg's argument
   +-+-------+--------+--- ... ---+

 */

#define BLK_HDR_SIZE 2

typedef unsigned char t_msg;

#define M_NUL  0			    /* Ignored by binkp (data
					     * optionally Logged) */
#define M_ADR  1
#define M_PWD  2
#define M_FILE 3
#define M_OK   4			    /* The password is ok (data
					     * ignored) */
#define M_EOB  5			    /* End-of-batch (data ignored) */
#define M_GOT  6			    /* File received */
#define M_ERR  7			    /* Misc errors */
#define M_BSY  8			    /* All AKAs are busy */
#define M_GET  9			    /* Get a file from offset */
#define M_SKIP 10			    /* Skip a file */
#define M_MAX  10

typedef struct _BMSG BMSG;
struct _BMSG
{
  t_msg t;				    /* msg type */
  int sz;				    /* msg size */
  char *s;				    /* msg text */
};

/* A file in transfer */
typedef struct _TFILE TFILE;
struct _TFILE
{
  char path[MAXPATHLEN + 1];		    /* It's name */
  char flo[MAXPATHLEN + 1];		    /* It's .?lo */
  char netname[MAX_NETNAME + 1];	    /* It's "netname" */
  char action;				    /* 'd'elete, 't'runcate, '\0' -- *
					     * none */
  char type;
  unsigned long size;
  time_t start;				    /* The transfer started at... */
  time_t time;				    /* Filetime */
  FILE *f;
};

#define TF_ZERO(a) (memset(a, 0, sizeof(*a)))

/* Protocol's state */
typedef struct _STATE STATE;
struct _STATE
{
  SOCKET s;
  FTN_NODE *to;				    /* Dest. address (if an outbound
					     * connection) */

  /* binkp buffres */
  char *obuf;
  int osize;				    /* -1=header NOT sent */
  int oread;				    /* 1=Data already read, now send */

  char *ibuf;
  int isize;				    /* Current block's size.
					     * -1=expecting block header */
  int iread;				    /* Number of bytes read already */
  int imsg;				    /* 0=data block,
					     * 1=message(command) */

  /* binkp queues and data */
  BMSG *msgs;				    /* Output msg queue */
  int n_msgs;
  TFILE in, out;			    /* Files in transfer */
  TFILE flo;				    /* A .?lo in transfer */
  TFILE *sent_fls;			    /* Sent files: waiting for GOT */
  int n_sent_fls;			    /* The number of... */
  FTNQ *q;				    /* Queue */
  FTN_ADDR *fa;				    /* Foreign akas */
  int nfa;				    /* How many... */
  int maxflvr;

  /* binkp state description */
  int local_EOB, remote_EOB;
  int GET_FILE_balance;			    /* Prevent quitting without
					     * waiting for GET results */
  int state;
};

#define OBLKSIZE oblksize
